import datetime
import os
import unittest

from . import TESTDIR, TESTDATADIR
from .. import Article, Corpus, get_corpus_dir, starterdir

from ..transformations import (doi_to_path, url_to_path, filename_to_doi, url_to_doi,
                               filename_to_url, doi_to_url, INT_URL_TMP, EXT_URL_TMP)
from ..plos_corpus import listdir_nohidden


suffix = '.xml'
example_url = 'http://journals.plos.org/plosone/article/file?id=10.1371/'\
              'journal.pbio.2001413&type=manuscript'
example_url_int = 'http://contentrepo.plos.org:8002/v1/objects/mogilefs-prod-'\
                  'repo?key=10.1371/journal.pbio.2001413.XML'
example_file = 'journal.pbio.2001413.xml'
example_doi = '10.1371/journal.pbio.2001413'
example_url2 = 'http://journals.plos.org/plosone/article/file?id=10.1371/'\
               'annotation/3155a3e9-5fbe-435c-a07a-e9a4846ec0b6&type=manuscript'
example_url2_int = 'http://contentrepo.plos.org:8002/v1/objects/mogilefs-'\
                   'prod-repo?key=10.1371/annotation/3155a3e9-5fbe-435c-a'\
                   '07a-e9a4846ec0b6.XML'
example_file2 = 'plos.correction.3155a3e9-5fbe-435c-a07a-e9a4846ec0b6.xml'
example_doi2 = '10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846ec0b6'
class_doi = '10.1371/journal.pone.0185809'


class TestDOIMethods(unittest.TestCase):

    def test_doi_conversions(self):
        """
        TODO: What this tests are about!
        """
        self.assertEqual(os.path.join(get_corpus_dir(), example_file), doi_to_path(example_doi), "{0} does not transform to {1}".format(example_doi, example_file))
        self.assertEqual(example_file2, doi_to_path(example_doi2, ''), "{0} does not transform to {1}".format(example_doi2, example_file2))
        self.assertEqual(example_url2, doi_to_url(example_doi2), "{0} does not transform to {1}".format(example_doi2, example_url2))
        self.assertEqual(example_url, doi_to_url(example_doi), "In doi_to_url, {0} does not transform to {1}".format(example_doi, example_url))
        self.assertEqual(example_url2_int, doi_to_url(example_doi2, plos_network=True),
                         "In doi_to_url, {0} does not transform to {1}, but to {2}".format(example_doi2,
                         example_url2_int, doi_to_url(example_doi2)))
        self.assertEqual(example_url_int, doi_to_url(example_doi, plos_network=True),
                         "{0} does not transform to {1}".format(example_doi, example_url_int))

    def test_file_conversions(self):
        """
        TODO: What this tests are about!
        """
        self.assertEqual(example_doi, filename_to_doi(example_file),
                         "{0} does not transform to {1}".format(example_file, example_doi))
        self.assertEqual(example_doi2, filename_to_doi(example_file2),
                         "{0} does not transform to {1}".format(example_file2, example_doi2))
        self.assertEqual(example_url, filename_to_url(example_file),
                         "{0} does not transform to {1}".format(example_file, example_url))
        self.assertEqual(example_url2, filename_to_url(example_file2),
                         "{0} does not transform to {1}".format(example_file2, example_url2))
        self.assertEqual(example_url_int, filename_to_url(example_file,
                         plos_network=True),
                         "{0} does not transform to {1}".format(example_file,
                         example_url_int))
        self.assertEqual(example_url2_int, filename_to_url(example_file2,
                         plos_network=True),
                         "{0} does not transform to {1}".format(example_file2,
                         example_url2))


    def test_url_conversions(self):
        """
        TODO: What this tests are about!
        """
        self.assertEqual(example_doi, url_to_doi(example_url),
                         "{0} does not transform to {1}".format(example_url, example_doi))
        self.assertEqual(example_doi2, url_to_doi(example_url2),
                         "{0} does not transform to {1}".format(example_url2, example_doi2))
        self.assertEqual(example_file, url_to_path(example_url, ''),
                         "{0} does not transform to {1}".format(example_url, example_file))
        self.assertEqual(example_file2, url_to_path(example_url2, ''),
                         "{0} does not transform to {1}".format(example_url2, example_file2))
        self.assertEqual(example_doi, url_to_doi(example_url_int),
                         "{0} does not transform to {1}".format(example_url_int, example_doi))
        self.assertEqual(example_doi2, url_to_doi(example_url2_int),
                         "{0} does not transform to {1}".format(example_url2_int, example_doi2))
        self.assertEqual(example_file, url_to_path(example_url_int, ''),
                         "{0} does not transform to {1}".format(example_url_int, example_file))
        # Test temporary commented out.
        # self.assertEqual(example_file2, url_to_path(example_url2_int, plos_network=True),
        #                 "{0} does not transform to {1}".format(example_url2_int, example_file2))


class TestArticleClass(unittest.TestCase):

    def test_class_doi1(self):
        """Tests the methods and properties of the Article class
        Test article DOI: 10.1371/journal.pone.0185809
        TODO: there is a socket warning from requests module. See https://github.com/requests/requests/issues/3912
        XML file is in test directory
        """
        article = Article(class_doi, directory=TESTDATADIR)
        self.assertEqual(article.check_if_doi_resolves(), "works", 'check_if_doi_resolves does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.check_if_link_works(), True, 'check_if_link_works does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, False, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_aff_dict(), {'aff001': 'Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands', 'aff002': 'Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany', 'aff003': 'University of Sussex, School of Life Sciences, Falmer, Brighton BN1 9QG, United Kingdom', 'edit1': 'University of Saskatchewan, CANADA'}, 'get_aff_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributions_dict(), {}, 'get_contributions_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributors_info(), [{'contrib_initials': 'CAH', 'given_names': 'Caspar A.', 'surname': 'Hallmann', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-4630-0522', 'authenticated': 'true'}], 'rid_dict': {'corresp': ['cor001'], 'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['c.hallmann@science.ru.nl'], 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Formal analysis', 'Investigation', 'Methodology', 'Software', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'MS', 'given_names': 'Martin', 'surname': 'Sorg', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Funding acquisition', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'EJ', 'given_names': 'Eelke', 'surname': 'Jongejans', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Funding acquisition', 'Investigation', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Henk', 'surname': 'Siepel', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Investigation', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'NH', 'given_names': 'Nick', 'surname': 'Hofland', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Formal analysis', 'Resources', 'Software', 'Validation']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Heinz', 'surname': 'Schwan', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Funding acquisition', 'Investigation', 'Methodology', 'Project administration', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'WS', 'given_names': 'Werner', 'surname': 'Stenmans', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Funding acquisition', 'Methodology', 'Project administration', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'AM', 'given_names': 'Andreas', 'surname': 'Müller', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Project administration', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Hubert', 'surname': 'Sumser', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'TH', 'given_names': 'Thomas', 'surname': 'Hörren', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'DG', 'given_names': 'Dave', 'surname': 'Goulson', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff003']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['University of Sussex, School of Life Sciences, Falmer, Brighton BN1 9QG, United Kingdom'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Investigation', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HK', 'given_names': 'Hans', 'surname': 'de Kroon', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Funding acquisition', 'Investigation', 'Methodology', 'Resources', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'EGL', 'given_names': 'Eric Gordon', 'surname': 'Lamb', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['edit1']}, 'contrib_type': 'editor', 'author_type': None, 'editor_type': None, 'email': None, 'affiliations': ['University of Saskatchewan, CANADA'], 'author_roles': {None: ['Editor']}, 'footnotes': []}], 'get_contributors_info does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_corr_author_emails(), {'cor001': ['c.hallmann@science.ru.nl']}, 'get_corr_author_emails does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_dates(), {'collection': datetime.datetime(2017, 1, 1, 0, 0), 'epub': datetime.datetime(2017, 10, 18, 0, 0), 'received': datetime.datetime(2017, 7, 28, 0, 0), 'accepted': datetime.datetime(2017, 9, 19, 0, 0), 'updated': ''}, 'get_dates does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_fn_dict(), {'coi001': 'The authors have declared that no competing interests exist.'}, 'get_fn_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_plos_journal(), "PLOS ONE", 'get_plos_journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_related_dois(), {}, 'get_related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_url(), "http://journals.plos.org/plosone/article/file?id=10.1371/journal.pone.0185809&type=manuscript", 'get_url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.abstract[:100], "Global declines in insects have sparked wide interest among scientists, politicians, and the general", 'abstract does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.authors, [{'contrib_initials': 'CAH', 'given_names': 'Caspar A.', 'surname': 'Hallmann', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-4630-0522', 'authenticated': 'true'}], 'rid_dict': {'corresp': ['cor001'], 'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['c.hallmann@science.ru.nl'], 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Formal analysis', 'Investigation', 'Methodology', 'Software', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'MS', 'given_names': 'Martin', 'surname': 'Sorg', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Funding acquisition', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'EJ', 'given_names': 'Eelke', 'surname': 'Jongejans', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Funding acquisition', 'Investigation', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Henk', 'surname': 'Siepel', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Investigation', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'NH', 'given_names': 'Nick', 'surname': 'Hofland', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Formal analysis', 'Resources', 'Software', 'Validation']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Heinz', 'surname': 'Schwan', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Funding acquisition', 'Investigation', 'Methodology', 'Project administration', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'WS', 'given_names': 'Werner', 'surname': 'Stenmans', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Funding acquisition', 'Methodology', 'Project administration', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'AM', 'given_names': 'Andreas', 'surname': 'Müller', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Project administration', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HS', 'given_names': 'Hubert', 'surname': 'Sumser', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'TH', 'given_names': 'Thomas', 'surname': 'Hörren', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Entomological Society Krefeld e.V., Entomological Collections Krefeld, Marktstrasse 159, 47798 Krefeld, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Data curation', 'Investigation', 'Methodology', 'Resources', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'DG', 'given_names': 'Dave', 'surname': 'Goulson', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff003']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['University of Sussex, School of Life Sciences, Falmer, Brighton BN1 9QG, United Kingdom'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Investigation', 'Writing – review & editing']}, 'footnotes': []}, {'contrib_initials': 'HK', 'given_names': 'Hans', 'surname': 'de Kroon', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Funding acquisition', 'Investigation', 'Methodology', 'Resources', 'Supervision', 'Writing – review & editing']}, 'footnotes': []}], 'authors does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.corr_author, [{'contrib_initials': 'CAH', 'given_names': 'Caspar A.', 'surname': 'Hallmann', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-4630-0522', 'authenticated': 'true'}], 'rid_dict': {'corresp': ['cor001'], 'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['c.hallmann@science.ru.nl'], 'affiliations': ['Radboud University, Institute for Water and Wetland Research, Animal Ecology and Physiology & Experimental Plant Ecology, PO Box 9100, 6500 GL Nijmegen, The Netherlands'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Formal analysis', 'Investigation', 'Methodology', 'Software', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review & editing']}, 'footnotes': []}], 'corr_author does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, False, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.counts, {'fig-count': '5', 'table-count': '4', 'page-count': '21'}, 'counts does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.doi, "10.1371/journal.pone.0185809", 'doi does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.dtd, "JATS 1.1d3", 'dtd does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.editor, [{'contrib_initials': 'EGL', 'given_names': 'Eric Gordon', 'surname': 'Lamb', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['edit1']}, 'contrib_type': 'editor', 'author_type': None, 'editor_type': None, 'email': None, 'affiliations': ['University of Saskatchewan, CANADA'], 'author_roles': {None: ['Editor']}, 'footnotes': []}], 'editor does not transform correctly for {}'.format(article.doi))
        article_relpath = os.path.relpath(article.filename, TESTDIR)
        self.assertEqual(article_relpath, "testdata/journal.pone.0185809.xml", 'filename does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.journal, "PLOS ONE", 'journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.local, True, 'local does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.page, "http://journals.plos.org/plosone/article?id=10.1371/journal.pone.0185809", 'page does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.plostype, "Research Article", 'plostype does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.proof, '', 'proof does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.pubdate, datetime.datetime(2017, 10, 18, 0, 0), 'pubdate does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.related_dois, [], 'related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.title, "More than 75 percent decline over 27 years in total flying insect biomass in protected areas", 'title does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.type_, "research-article", 'type_ does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.url, "http://journals.plos.org/plosone/article/file?id=10.1371/journal.pone.0185809&type=manuscript", 'url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.word_count, 6646, 'word_count does not transform correctly for {}'.format(article.doi))

    def test_example_doi(self):
        """Tests the methods and properties of the Article class
        Test article DOI: 10.1371/journal.pbio.2001413
        XML file is in test directory
        """
        article = Article(example_doi, directory=TESTDATADIR)
        self.assertEqual(article.check_if_doi_resolves(), "works", 'check_if_doi_resolves does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.check_if_link_works(), True, 'check_if_link_works does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, False, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_aff_dict(), {'aff001': 'Department of Bioengineering, Stanford University, Stanford, California, United States of America', 'aff002': 'Isaac Newton Graham Middle School, Mountain View, California, United States of America', 'aff003': 'MYP Dresden International School, Dresden, Germany', 'aff004': 'University of California Santa Cruz, Santa Cruz, California, United States of America', 'aff005': 'Georgia Institute of Technology, Atlanta, Georgia, United States of America'}, 'get_aff_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributions_dict(), {}, 'get_contributions_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributors_info(), [{'contrib_initials': 'LCG', 'given_names': 'Lukas C.', 'surname': 'Gerber', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Department of Bioengineering, Stanford University, Stanford, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Investigation', 'Methodology', 'Resources', 'Software', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review and editing']}, 'footnotes': []}, {'contrib_initials': 'ACK', 'given_names': 'Agnes', 'surname': 'Calasanz-Kaiser', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Isaac Newton Graham Middle School, Mountain View, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Investigation', 'Resources', 'Software', 'Supervision']}, 'footnotes': []}, {'contrib_initials': 'LH', 'given_names': 'Luke', 'surname': 'Hyman', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff003']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['MYP Dresden International School, Dresden, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'KV', 'given_names': 'Kateryna', 'surname': 'Voitiuk', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff004']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['University of California Santa Cruz, Santa Cruz, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'UP', 'given_names': 'Uday', 'surname': 'Patil', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff005']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Georgia Institute of Technology, Atlanta, Georgia, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'IHRK', 'given_names': 'Ingmar H.', 'surname': 'Riedel-Kruse', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-6068-5561', 'authenticated': 'true'}], 'rid_dict': {'aff': ['aff001'], 'corresp': ['cor001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['ingmar@stanford.edu'], 'affiliations': ['Department of Bioengineering, Stanford University, Stanford, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Funding acquisition', 'Investigation', 'Methodology', 'Project administration', 'Resources', 'Software', 'Supervision', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review and editing']}, 'footnotes': []}], 'get_contributors_info does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_corr_author_emails(), {'cor001': ['ingmar@stanford.edu']}, 'get_corr_author_emails does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_dates(), {'epub': datetime.datetime(2017, 3, 21, 0, 0), 'collection': datetime.datetime(2017, 3, 1, 0, 0), 'updated': ''}, 'get_dates does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_fn_dict(), {'coi001': 'The authors have declared that no competing interests exist.'}, 'get_fn_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_plos_journal(), "PLOS Biology", 'get_plos_journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_related_dois(), {}, 'get_related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_url(), "http://journals.plos.org/plosone/article/file?id=10.1371/journal.pbio.2001413&type=manuscript", 'get_url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.abstract[:100], "Liquid-handling robots have many applications for biotechnology and the life sciences, with increasi", 'abstract does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.authors, [{'contrib_initials': 'LCG', 'given_names': 'Lukas C.', 'surname': 'Gerber', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff001']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Department of Bioengineering, Stanford University, Stanford, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Investigation', 'Methodology', 'Resources', 'Software', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review and editing']}, 'footnotes': []}, {'contrib_initials': 'ACK', 'given_names': 'Agnes', 'surname': 'Calasanz-Kaiser', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff002']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Isaac Newton Graham Middle School, Mountain View, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Investigation', 'Resources', 'Software', 'Supervision']}, 'footnotes': []}, {'contrib_initials': 'LH', 'given_names': 'Luke', 'surname': 'Hyman', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff003']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['MYP Dresden International School, Dresden, Germany'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'KV', 'given_names': 'Kateryna', 'surname': 'Voitiuk', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff004']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['University of California Santa Cruz, Santa Cruz, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'UP', 'given_names': 'Uday', 'surname': 'Patil', 'group_name': None, 'ids': [], 'rid_dict': {'aff': ['aff005']}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': ['Georgia Institute of Technology, Atlanta, Georgia, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Investigation', 'Software']}, 'footnotes': []}, {'contrib_initials': 'IHRK', 'given_names': 'Ingmar H.', 'surname': 'Riedel-Kruse', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-6068-5561', 'authenticated': 'true'}], 'rid_dict': {'aff': ['aff001'], 'corresp': ['cor001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['ingmar@stanford.edu'], 'affiliations': ['Department of Bioengineering, Stanford University, Stanford, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Funding acquisition', 'Investigation', 'Methodology', 'Project administration', 'Resources', 'Software', 'Supervision', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review and editing']}, 'footnotes': []}], 'authors does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.corr_author, [{'contrib_initials': 'IHRK', 'given_names': 'Ingmar H.', 'surname': 'Riedel-Kruse', 'group_name': None, 'ids': [{'id_type': 'orcid', 'id': 'http://orcid.org/0000-0002-6068-5561', 'authenticated': 'true'}], 'rid_dict': {'aff': ['aff001'], 'corresp': ['cor001']}, 'contrib_type': 'author', 'author_type': 'corresponding', 'editor_type': None, 'email': ['ingmar@stanford.edu'], 'affiliations': ['Department of Bioengineering, Stanford University, Stanford, California, United States of America'], 'author_roles': {'CASRAI CREDiT taxonomy': ['Conceptualization', 'Data curation', 'Formal analysis', 'Funding acquisition', 'Investigation', 'Methodology', 'Project administration', 'Resources', 'Software', 'Supervision', 'Validation', 'Visualization', 'Writing – original draft', 'Writing – review and editing']}, 'footnotes': []}], 'corr_author does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, False, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.counts, {'fig-count': '3', 'table-count': '0', 'page-count': '9'}, 'counts does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.doi, "10.1371/journal.pbio.2001413", 'doi does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.dtd, "JATS 1.1d3", 'dtd does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.editor, [], 'editor does not transform correctly for {}'.format(article.doi))
        article_relpath = os.path.relpath(article.filename, TESTDIR)
        self.assertEqual(article_relpath, "testdata/journal.pbio.2001413.xml", 'filename does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.journal, "PLOS Biology", 'journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.local, True, 'local does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.page, "http://journals.plos.org/plosbiology/article?id=10.1371/journal.pbio.2001413", 'page does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.plostype, "Community Page", 'plostype does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.proof, '', 'proof does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.pubdate, datetime.datetime(2017, 3, 21, 0, 0), 'pubdate does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.related_dois, [], 'related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.title, "Liquid-handling Lego robots and experiments for STEM education and research", 'title does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.type_, "other", 'type_ does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.url, "http://journals.plos.org/plosone/article/file?id=10.1371/journal.pbio.2001413&type=manuscript", 'url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.word_count, 3070, 'word_count does not transform correctly for {}'.format(article.doi))

    def test_example_doi2(self):
        """Tests the methods and properties of the Article class
        Test article DOI: 10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846ec0b6
        XML file is in test directory
        """
        article = Article(example_doi2, directory=TESTDATADIR)
        self.assertEqual(article.check_if_doi_resolves(), "works", 'check_if_doi_resolves does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.check_if_link_works(), True, 'check_if_link_works does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, True, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_aff_dict(), {}, 'get_aff_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributions_dict(), {}, 'get_contributions_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_contributors_info(), [{'contrib_initials': 'LW', 'given_names': 'LingLin', 'surname': 'Wan', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'JH', 'given_names': 'Juan', 'surname': 'Han', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'MS', 'given_names': 'Min', 'surname': 'Sang', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'AL', 'given_names': 'AiFen', 'surname': 'Li', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'HW', 'given_names': 'Hong', 'surname': 'Wu', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'SY', 'given_names': 'ShunJi', 'surname': 'Yin', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'CZ', 'given_names': 'ChengWu', 'surname': 'Zhang', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}], 'get_contributors_info does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_corr_author_emails(), {}, 'get_corr_author_emails does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_dates(), {'collection': datetime.datetime(2012, 1, 1, 0, 0), 'epub': datetime.datetime(2012, 6, 29, 0, 0), 'updated': ''}, 'get_dates does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_fn_dict(), {}, 'get_fn_dict does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_plos_journal(), "PLOS ONE", 'get_plos_journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_related_dois(), {'retracted-article': ['10.1371/journal.pone.0035142']}, 'get_related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.get_url()[:100], "http://journals.plos.org/plosone/article/file?id=10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846e", 'get_url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.abstract, "", 'abstract does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.authors, [{'contrib_initials': 'LW', 'given_names': 'LingLin', 'surname': 'Wan', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'JH', 'given_names': 'Juan', 'surname': 'Han', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'MS', 'given_names': 'Min', 'surname': 'Sang', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'AL', 'given_names': 'AiFen', 'surname': 'Li', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'HW', 'given_names': 'Hong', 'surname': 'Wu', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'SY', 'given_names': 'ShunJi', 'surname': 'Yin', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}, {'contrib_initials': 'CZ', 'given_names': 'ChengWu', 'surname': 'Zhang', 'group_name': None, 'ids': [], 'rid_dict': {}, 'contrib_type': 'author', 'author_type': 'contributing', 'editor_type': None, 'email': None, 'affiliations': [], 'author_roles': {}, 'footnotes': []}], 'authors does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.corr_author, [], 'corr_author does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.amendment, True, 'amendment does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.counts, {}, 'counts does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.doi, "10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846ec0b6", 'doi does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.dtd, "NLM 3.0", 'dtd does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.editor, [], 'editor does not transform correctly for {}'.format(article.doi))
        article_relpath = os.path.relpath(article.filename, TESTDIR)
        self.assertEqual(article_relpath, "testdata/plos.correction.3155a3e9-5fbe-435c-a07a-e9a4846ec0b6.xml", 'filename does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.journal, "PLOS ONE", 'journal does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.local, True, 'local does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.page, "http://journals.plos.org/plosone/article?id=10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846ec0b6", 'page does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.plostype, "Retraction", 'plostype does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.proof, '', 'proof does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.pubdate, datetime.datetime(2012, 6, 29, 0, 0), 'pubdate does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.related_dois, ["10.1371/journal.pone.0035142"], 'related_dois does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.title[:100], "Retraction: De Novo Transcriptomic Analysis of an Oleaginous Microalga: Pathway Description and Gene", 'title does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.type_, "retraction", 'type_ does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.url[:100], "http://journals.plos.org/plosone/article/file?id=10.1371/annotation/3155a3e9-5fbe-435c-a07a-e9a4846e", 'url does not transform correctly for {}'.format(article.doi))
        self.assertEqual(article.word_count, 129, 'word_count does not transform correctly for {}'.format(article.doi))


class TestCorpusClass(unittest.TestCase):
    def test_starterdir(self):
        corpus = Corpus(directory=starterdir)
        self.assertEqual(len(corpus.files), len(listdir_nohidden(starterdir)), 'Number of files in corpus incorrect')
        self.assertEqual(set(corpus.filepaths), set(listdir_nohidden(starterdir)), "Files in corpus incorrect")
        self.assertTrue('journal.pcbi.0030158.xml' in corpus.files)
        self.assertTrue('10.1371/journal.pmed.0030132' in corpus.dois)

if __name__ == "__main__":
    unittest.main()



from collections import OrderedDict
import os
import random

from . import get_corpus_dir, Article
from .transformations import filename_to_doi, doi_to_path


class Corpus():
    """A collection of PLOS articles."""

    def __init__(self, directory=None, plos_network=False, extension='.xml'):
        """Creation of an article corpus class."""
        if directory is None:
            directory = get_corpus_dir()
        self.directory = directory
        self.plos_network = plos_network
        self.extension = extension

    def __repr__(self):
        """Value of a corpus object when you call it directly on the command line.

        Shows the directory location of the corpus
        :returns: directory
        :rtype: {str}
        """
        out = "Corpus location: {0}\nNumber of files: {1}".format(self.directory, len(self.files))
        return out

    @property
    def iter_file_doi(self):
        """Generator that returns filename, doi tuples for every file in the corpus.

        Used to generate both DOI and file generators for the corpus. 
        """
        return ((file_, filename_to_doi(file_))
                for file_ in os.listdir(self.directory)
                if file_.endswith(self.extension) and 'DS_Store' not in file_)

    @property
    def file_doi(self):
        """An ordered dict that maps every corpus file to its accompanying DOI."""
        return OrderedDict(self.iter_file_doi)

    @property
    def iter_files(self):
        """Generator of article XML filenames in the corpus directory."""

        return (x[0] for x in self.iter_file_doi)

    @property
    def iter_dois(self):
        """Generator of DOIs of the articles in the corpus directory.

        Use for looping through all corpus articles with the Article class.
        """

        return (x[1] for x in self.iter_file_doi)

    @property
    def iter_filepaths(self):
        """Generator of article XML files in corpus directory, including the full path."""
        return (os.path.join(self.directory, fname) for fname in self.iter_files)

    @property
    def files(self):
        """List of article XML files in the corpus directory."""

        return list(self.iter_files)

    @property
    def dois(self):
        """List of DOIs of the articles in the corpus directory."""

        return list(self.iter_dois)

    @property
    def filepaths(self):
        """List of article XML files in corpus directory, including the full path."""
        return list(self.iter_filepaths)

    def random_dois(self, count):
        """
        Gets a list of random DOIs. Construct from local files in
        corpus directory. Length of list specified in `count` parameter.
        :param count: specify how many DOIs are to be returned
        :return: a list of random DOIs for analysis
        """

        return random.sample(self.dois, count)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Make a SQLite DB out of articles XML files
"""

import argparse
import datetime
import os
import random

from tqdm import tqdm
import sqlite3

from peewee import Model, CharField, ForeignKeyField, TextField, \
    DateTimeField, BooleanField, IntegerField, IntegrityError
from playhouse.sqlite_ext import SqliteExtDatabase

from .transformations import filename_to_doi, convert_country
from .article_class import Article


# TODO: this may need to be updated to take into account the new get_corpus_dir() logic.
# It is not clear, since this is a relative path from the package how this should work without
# diving into the code more thoroughly 
corpusdir = 'allofplos/allofplos_xml'

journal_title_dict = {
    'PLOS ONE': 'PLOS ONE',
    'PLOS GENETICS': 'PLOS Genetics',
    'PLOS GENET': 'PLOS Genetics',
    'PLOS NEGLECTED TROPICAL DISEASES': 'PLOS Neglected Tropical Diseases',
    'PLOS NEGL TROP DIS': 'PLOS Neglected Tropical Diseases',
    'PLOS BIOLOGY': 'PLOS Biology',
    'PLOS BIOL': 'PLOS Biology',
    'PLOS CLINICAL TRIALS': 'PLOS Clinical Trials',
    'PLOS MEDICINE':'PLOS Medicine',
    'PLOS MEDICIN': 'PLOS Medicine',
    'PLOS MED': 'PLOS Medicine',
    'PLOS PATHOG': 'PLOS Pathogens',
    'PLOS PATHOGENS': 'PLOS Pathogens',
    'PLOS COMPUTATIONAL BIOLOGY': 'PLOS Computational Biology',
    'PLOS COMPUT BIOL': 'PLOS Computational Biology',
    'PLOS CLINICAL TRIALS': 'PLOS Clinical Trials',
}


parser = argparse.ArgumentParser()
parser.add_argument('--db', action='store', help=
                    'Name the db', default='ploscorpus.db')
parser.add_argument('--random', action='store', type=int, help=
                    'Number of articles in a random subset. '
                    'By default will use all articles')
parser.add_argument('--starterdb', action='store_true', help=
                    'Make the starter database', dest='starter')
args = parser.parse_args()

# TODO: Put a warning that the DB will be deleted
if os.path.isfile(args.db):
    os.remove(args.db)

if args.starter:
    if os.path.isfile('starter.db'):
        os.remove('starter.db')
    db = SqliteExtDatabase('starter.db')
else:
    db = SqliteExtDatabase(args.db)

class BaseModel(Model):
    class Meta:
        database = db

class Journal(BaseModel):
    journal = CharField(unique=True)

class ArticleType(BaseModel):
    article_type = CharField(unique=True)

class Country(BaseModel):
    country = CharField(unique=True)

class Affiliations(BaseModel):
    affiliations = CharField(unique=True)

class CorrespondingAuthor(BaseModel):
    corr_author_email = CharField(unique=True)
    tld = TextField(null=True)
    given_name = TextField(null=True)
    surname = TextField(null=True)
    group_name = TextField(null=True)
    affiliation = ForeignKeyField(Affiliations, related_name='aff')
    country = ForeignKeyField(Country, related_name='aff')

class JATSType(BaseModel):
    jats_type = CharField(unique=True)

class PLOSArticle(BaseModel):
    DOI = TextField(unique=True)
    abstract = TextField()
    title = TextField()
    plostype = ForeignKeyField(ArticleType, related_name='arttype')
    journal = ForeignKeyField(Journal, related_name='journals')
    created_date = DateTimeField(default=datetime.datetime.now)
    word_count = IntegerField()
    JATS_type = ForeignKeyField(JATSType, related_name='jats')

class CoAuthorPLOSArticle(BaseModel):
    corr_author = ForeignKeyField(CorrespondingAuthor)
    article = ForeignKeyField(PLOSArticle)

db.connect()
db.create_tables([Journal, PLOSArticle, ArticleType,
                      CoAuthorPLOSArticle, CorrespondingAuthor,
                      JATSType, Affiliations, Country])

if args.starter:
    allfiles = os.listdir('starter_corpus')
else:
    allfiles = os.listdir(corpusdir)
if args.random:
    randomfiles = random.sample(allfiles, args.random)
    max_value = len(randomfiles)
else:
    max_value = len(allfiles)

for i, file_ in enumerate(tqdm(randomfiles if args.random else allfiles)):
    doi = filename_to_doi(file_)
    article = Article(doi)
    journal_name = journal_title_dict[article.journal.upper()]
    with db.atomic() as atomic:
        try:
            journal = Journal.create(journal = journal_name)
        except IntegrityError:
            db.rollback()
            journal = Journal.get(Journal.journal == journal_name)
    with db.atomic() as atomic:
        try:
            article_type = ArticleType.create(article_type = article.plostype)
        except IntegrityError:
            db.rollback()
            article_type = ArticleType.get(ArticleType.article_type == article.plostype)
    with db.atomic() as atomic:
        try:
            j_type = JATSType.create(jats_type = article.type_)
        except IntegrityError:
            db.rollback()
            j_type = JATSType.get(JATSType.jats_type == article.type_)
    p_art = PLOSArticle.create(
        DOI = doi,
        journal = journal,
        abstract=article.abstract.replace('\n', '').replace('\t', ''),
        title = article.title.replace('\n', '').replace('\t', ''),
        plostype = article_type,
        created_date = article.pubdate,
        word_count=article.word_count,
        JATS_type = j_type)
    if article.authors:
        iterable_authors = article.authors
    else:
        iterable_authors = []
    for auths in iterable_authors:
        if auths['email']:
            with db.atomic() as atomic:
                if auths['affiliations']:
                    author_aff = auths['affiliations'][0]
                else:
                    author_aff = 'N/A'
                try:
                    aff = Affiliations.create(affiliations = author_aff)
                except (sqlite3.IntegrityError, IntegrityError):
                    db.rollback()
                    aff = Affiliations.get(Affiliations.affiliations ==
                                           author_aff)
            with db.atomic() as atomic:
                try:
                    if auths['affiliations'][0] == '':
                        country_from_aff = 'N/A'
                    else:
                        country_from_aff = auths['affiliations'][0].\
                                           split(',')[-1].strip()
                except IndexError:
                    country_from_aff = 'N/A'
                country_from_aff = convert_country(country_from_aff)
                try:
                    country = Country.create(country = country_from_aff)
                except IntegrityError:
                    db.rollback()
                    country = Country.get(Country.country == country_from_aff)

            try:
                co_author = CorrespondingAuthor.create(
                    corr_author_email = auths['email'][0],
                    tld = auths['email'][0].split('.')[-1],
                    given_name = auths['given_names'],
                    surname = auths['surname'],
                    group_name = auths['group_name'],
                    affiliation = aff,
                    country = country
                    )
            except IntegrityError:
                co_author = CorrespondingAuthor.\
                            get(CorrespondingAuthor.corr_author_email == auths['email'][0])
            coauthplosart = CoAuthorPLOSArticle.create(
                    corr_author = co_author,
                    article = p_art
                )



from setuptools import setup, find_packages
from os import path
import sys

if sys.version_info.major < 3:
    sys.exit('Sorry, Python < 3.4 is not supported')
elif sys.version_info.minor < 4:
    sys.exit('Sorry, Python < 3.4 is not supported')

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='allofplos',
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.10.0',
    description='Get and analyze all PLOS articles',
    long_description=long_description,
    url='https://github.com/PLOS/allofplos',
    author='Elizabeth Seiver, Sebastian Bassi, M Pacer',
    author_email='eseiver@plos.org, sebastian.bassi@globant.com, mpacer@berkeley.edu',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='science PLOS publishing',
    packages=find_packages(),
    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=[
        'certifi==2017.7.27.1',
        'chardet>=3.0.4',
        'idna>=2.6',
        'lxml>=4.0.0',
        'peewee==2.10.2',
        'python-utils>=2.2.0',
        'requests>=2.18.4',
        'six>=1.11.0',
        'tqdm==4.17.1',
        'urllib3==1.22',
        'unidecode==0.04.21',
        ],
    python_requires='>=3.4',
    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'allofplos': ['starter.db'],
        'allofplos.starter_corpus': ['*.xml'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    ##data_files=[('my_data', ['data/data_file'])],

    entry_points={
        'console_scripts': [
            'plos_corpus=allofplos.plos_corpus:main',
        ],
    },
)



import os

# path to the root of allofplos (the package)
ALLOFPLOS_DIR_PATH = os.path.abspath(os.path.dirname(__file__))

# Main directory for the corpus of article XML files
corpusdir = os.path.join(ALLOFPLOS_DIR_PATH, 'allofplos_xml')

# Starter pack of PLOS articles
starterdir = os.path.join(ALLOFPLOS_DIR_PATH, 'starter_corpus')

# Temporary folder for downloading and processing new articles
newarticledir = os.path.join(ALLOFPLOS_DIR_PATH, 'new_plos_articles')

# List of uncorrected proof articles to check for updates
uncorrected_proofs_text_list = os.path.join(ALLOFPLOS_DIR_PATH, 'uncorrected_proofs_list.txt')

def get_corpus_dir():
    """If you want to set the corpus directory, assign the desired path to 
    ``os.environ['PLOS_CORPUS']``.
    """
    import os
    return os.path.expanduser(os.environ.get("PLOS_CORPUS", "")) or corpusdir

del os

    

# NB: any packages that you want to expose at the top level, you will need to
# import after creating global variables that they may rely upon
# (e.g., corpusdir)

from .article_class import Article
from .corpus_class import Corpus



""" This is an example on how to convert the SQLite DB into a class model to
    make queries without using SQL syntax. This example uses peewee as ORM.
    It should be used as a template, the end user could edit it to fit their
    needs.
    TODO: Add more sample queries.
"""

from peewee import *

database = SqliteDatabase('ploscorpus.db', **{})

class UnknownField(object):
    def __init__(self, *_, **__): pass

class BaseModel(Model):
    class Meta:
        database = database

class Affiliations(BaseModel):
    affiliations = CharField(unique=True)

    class Meta:
        db_table = 'affiliations'

class Articletype(BaseModel):
    article_type = CharField(unique=True)

    class Meta:
        db_table = 'articletype'

class Jatstype(BaseModel):
    jats_type = CharField(unique=True)

    class Meta:
        db_table = 'jatstype'

class Journal(BaseModel):
    journal = CharField(unique=True)

    class Meta:
        db_table = 'journal'

class Plosarticle(BaseModel):
    doi = TextField(db_column='DOI', unique=True)
    jats_type = ForeignKeyField(db_column='JATS_type_id', rel_model=Jatstype, to_field='id')
    abstract = TextField()
    created_date = DateTimeField()
    journal = ForeignKeyField(db_column='journal_id', rel_model=Journal, to_field='id')
    plostype = ForeignKeyField(db_column='plostype_id', rel_model=Articletype, to_field='id')
    title = TextField()
    word_count = IntegerField()

    class Meta:
        db_table = 'plosarticle'

class Country(BaseModel):
    country = CharField(unique=True)

    class Meta:
        db_table = 'country'

class Correspondingauthor(BaseModel):
    affiliation = ForeignKeyField(db_column='affiliation_id', rel_model=Affiliations, to_field='id')
    corr_author_email = CharField(unique=True)
    country = ForeignKeyField(db_column='country_id', rel_model=Country, to_field='id')
    given_name = TextField(null=True)
    group_name = TextField(null=True)
    surname = TextField(null=True)
    tld = TextField(null=True)

    class Meta:
        db_table = 'correspondingauthor'

class Coauthorplosarticle(BaseModel):
    article = ForeignKeyField(db_column='article_id', rel_model=Plosarticle, to_field='id')
    corr_author = ForeignKeyField(db_column='corr_author_id', rel_model=Correspondingauthor, to_field='id')

    class Meta:
        db_table = 'coauthorplosarticle'



query = (Plosarticle
         .select()
         .join(Coauthorplosarticle)
         .join(Correspondingauthor)
         .join(Country)
         .join(Journal, on=(Plosarticle.journal == Journal.id))
         .where(Country.country == 'France')
         .where(Plosarticle.created_date > '2008-1-1')
         .where(Journal.journal == 'PLOS Computational Biology')
         )

for papers in query:
    print(papers.doi)
print(query.count())

query = (Plosarticle
         .select()
         .join(Coauthorplosarticle)
         .join(Correspondingauthor)
         .join(Affiliations)
         .join(Journal, on=(Plosarticle.journal == Journal.id))
         .where(Country.country == 'France')
         .where(Plosarticle.created_date > '2008-1-1')
         .where(Journal.journal == 'PLOS Computational Biology')
         )
